from pathlib import Path
from typing import Iterable, Union


class CiffTokenizer:
    def __init__(self, dictionary: Union[str, Path, Iterable]):
        if isinstance(dictionary, (str, Path)):
            with open(dictionary) as _file:
                self.dictionary = set(_file.read().splitlines())
        else:
            self.dictionary = set(dictionary)

        self.max_token_length = max(len(token) for token in self.dictionary)

    def tokenize(self, sentence):
        """
        Tokenize a sentence into pieces generically. The
        class should have a dict `self.dictionary` containing
        the tokens and an integer `self.max_token_length`
        containing the length of the longest token.
        """
        sentence = sentence.lower()

        sub_tokens = []
        begin = 0
        sentence_length = len(sentence)
        while begin < sentence_length:
            end = begin + self.max_token_length
            if end > sentence_length:
                end = sentence_length
            piece_found = None
            while begin < end:
                piece = sentence[begin:end]
                if piece in self.dictionary:
                    piece_found = piece
                    break
                end -= 1
            if piece_found:
                sub_tokens.append(piece)
                begin = end
            else:
                begin += 1

        return sub_tokens
